@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1999-2020 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,6 +24,9 @@
 #include "os-interface.h"
 #include "nv.h"
 #include "nv-linux.h"
+#include "nv-reg.h"
+
+extern NvU32 NVreg_EnableSystemMemoryPools;
 
 static inline void nv_set_contig_memory_uc(nvidia_pte_t *page_ptr, NvU32 num_pages)
 {
@@ -163,6 +166,12 @@ static inline void nv_set_memory_type(nv_alloc_t *at, NvU32 type)
     nvidia_pte_t *page_ptr;
     struct page *page;
 
+    if (at->flags.contig)
+    {
+        nv_set_contig_memory_type(&at->page_table[0], at->num_pages, type);
+        return;
+    }
+
     if (nv_set_memory_array_type_present(type))
     {
         status = os_alloc_mem((void **)&pages,
@@ -187,7 +196,7 @@ static inline void nv_set_memory_type(nv_alloc_t *at, NvU32 type)
     {
         for (i = 0; i < at->num_pages; i++)
         {
-            page_ptr = at->page_table[i];
+            page_ptr = &at->page_table[i];
             page = NV_GET_PAGE_STRUCT(page_ptr->phys_addr);
 #if defined(NV_SET_MEMORY_ARRAY_UC_PRESENT)
             pages[i] = (unsigned long)page_address(page);
@@ -211,7 +220,7 @@ static inline void nv_set_memory_type(nv_alloc_t *at, NvU32 type)
     else
     {
         for (i = 0; i < at->num_pages; i++)
-            nv_set_contig_memory_type(at->page_table[i], 1, type);
+            nv_set_contig_memory_type(&at->page_table[i], 1, type);
     }
 }
 
@@ -222,7 +231,7 @@ static NvU64 nv_get_max_sysmem_address(void)
 
     for_each_online_node(node_id)
     {
-        global_max_pfn = max(global_max_pfn, nv_node_end_pfn(node_id));
+        global_max_pfn = max(global_max_pfn, (NvU64)node_end_pfn(node_id));
     }
 
     return ((global_max_pfn + 1) << PAGE_SHIFT) - 1;
@@ -252,22 +261,18 @@ static unsigned int nv_compute_gfp_mask(
         if ((dev && dev->dma_mask && (*(dev->dma_mask) < max_sysmem_address)) ||
             (nv && nv->force_dma32_alloc))
         {
-            gfp_mask = NV_GFP_DMA32;
+            gfp_mask = NV_GFP_KERNEL | NV_GFP_DMA32;
         }
     }
-#if defined(__GFP_RETRY_MAYFAIL)
+
     gfp_mask |= __GFP_RETRY_MAYFAIL;
-#elif defined(__GFP_NORETRY)
-    gfp_mask |= __GFP_NORETRY;
-#endif
-#if defined(__GFP_ZERO)
+
     if (at->flags.zeroed)
         gfp_mask |= __GFP_ZERO;
-#endif
-#if defined(__GFP_THISNODE)
-    if (at->flags.node0)
+
+    if (at->flags.node)
         gfp_mask |= __GFP_THISNODE;
-#endif
+
     // Compound pages are required by vm_insert_page for high-order page
     // allocations
     if (at->order > 0)
@@ -276,6 +281,28 @@ static unsigned int nv_compute_gfp_mask(
     return gfp_mask;
 }
 
+// set subpages describing page
+static void
+nv_alloc_set_page
+(
+    nv_alloc_t *at,
+    unsigned int page_idx,
+    unsigned long virt_addr
+)
+{
+    unsigned long phys_addr = nv_get_kern_phys_address(virt_addr);
+    unsigned int os_pages_in_page = 1 << at->order;
+    unsigned int base_os_page = page_idx * os_pages_in_page;
+    unsigned int num_os_pages = NV_MIN(at->num_pages - base_os_page, os_pages_in_page);
+    unsigned int i;
+
+    for (i = 0; i < num_os_pages; i++)
+    {
+        at->page_table[base_os_page + i].virt_addr = virt_addr + i * PAGE_SIZE;
+        at->page_table[base_os_page + i].phys_addr = phys_addr + i * PAGE_SIZE;
+    }
+}
+
 /*
  * This function is needed for allocating contiguous physical memory in xen
  * dom0. Because of the use of xen sw iotlb in xen dom0, memory allocated by
@@ -294,15 +321,24 @@ static NV_STATUS nv_alloc_coherent_pages(
     NvU32 i;
     unsigned int gfp_mask;
     unsigned long virt_addr = 0;
-    dma_addr_t bus_addr;
-    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
-    struct device *dev = nvl->dev;
+    nv_linux_state_t *nvl;
+    struct device *dev;
+
+    if (!nv)
+    {
+        nv_printf(NV_DBG_MEMINFO,
+            "NVRM: VM: %s: coherent page alloc on nvidiactl not supported\n", __FUNCTION__);
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+    dev = nvl->dev;
 
     gfp_mask = nv_compute_gfp_mask(nv, at);
 
     virt_addr = (unsigned long)dma_alloc_coherent(dev,
                                                   at->num_pages * PAGE_SIZE,
-                                                  &bus_addr,
+                                                  &at->dma_handle,
                                                   gfp_mask);
     if (!virt_addr)
     {
@@ -313,26 +349,18 @@ static NV_STATUS nv_alloc_coherent_pages(
 
     for (i = 0; i < at->num_pages; i++)
     {
-        page_ptr = at->page_table[i];
+        page_ptr = &at->page_table[i];
 
         page_ptr->virt_addr = virt_addr + i * PAGE_SIZE;
         page_ptr->phys_addr = virt_to_phys((void *)page_ptr->virt_addr);
-        page_ptr->dma_addr  = bus_addr + i * PAGE_SIZE;
     }
 
     if (at->cache_type != NV_MEMORY_CACHED)
     {
-        nv_set_contig_memory_type(at->page_table[0],
-                                  at->num_pages,
-                                  NV_MEMORY_UNCACHED);
+        nv_set_memory_type(at, NV_MEMORY_UNCACHED);
     }
 
     at->flags.coherent = NV_TRUE;
-
-    // dma_alloc_coherent() allocates unencrypted memory when AMD SEV is enabled.
-    if (os_sev_enabled)
-        at->flags.unencrypted = NV_TRUE;
-
     return NV_OK;
 }
 
@@ -343,17 +371,205 @@ static void nv_free_coherent_pages(
     nvidia_pte_t *page_ptr;
     struct device *dev = at->dev;
 
-    page_ptr = at->page_table[0];
+    page_ptr = &at->page_table[0];
 
     if (at->cache_type != NV_MEMORY_CACHED)
     {
-        nv_set_contig_memory_type(at->page_table[0],
-                                  at->num_pages,
-                                  NV_MEMORY_WRITEBACK);
+        nv_set_memory_type(at, NV_MEMORY_WRITEBACK);
     }
 
     dma_free_coherent(dev, at->num_pages * PAGE_SIZE,
-                      (void *)page_ptr->virt_addr, page_ptr->dma_addr);
+                      (void *)page_ptr->virt_addr, at->dma_handle);
+}
+
+typedef struct
+{
+    unsigned long virt_addr;
+    struct list_head list_node;
+} nv_page_pool_entry_t;
+
+#define NV_MEM_POOL_LIST_HEAD(list) list_first_entry_or_null(list, nv_page_pool_entry_t, list_node)
+
+typedef struct nv_page_pool_t
+{
+    struct list_head clean_list;
+    struct list_head dirty_list;
+    nv_kthread_q_t scrubber_queue;
+    nv_kthread_q_item_t scrubber_queue_item;
+    int node_id;
+    unsigned int order;
+    unsigned long pages_owned;
+    void *lock;
+    struct shrinker *shrinker;
+
+#ifndef NV_SHRINKER_ALLOC_PRESENT
+    struct shrinker _shrinker;
+#endif
+} nv_page_pool_t;
+
+nv_page_pool_t *sysmem_page_pools[MAX_NUMNODES][NV_MAX_PAGE_ORDER + 1];
+
+NV_STATUS nv_init_page_pools(void) { return NV_OK; }
+void nv_destroy_page_pools(void) {}
+unsigned int nv_mem_pool_alloc_pages(nv_page_pool_t *mem_pool, nv_alloc_t *at) { return 0; }
+NV_STATUS nv_mem_pool_free_pages(nv_page_pool_t *mem_pool, nv_alloc_t *at) { return NV_OK; }
+
+static nv_page_pool_t *nv_mem_pool_get(int node_id, unsigned int order)
+{
+
+    if (node_id >= ARRAY_SIZE(sysmem_page_pools))
+        return NULL;
+
+    // get_order() is not limited by NV_MAX_PAGE_ORDER
+    if (order >= ARRAY_SIZE(sysmem_page_pools[node_id]))
+        return NULL;
+
+    return sysmem_page_pools[node_id][order];
+}
+
+void
+nv_free_system_pages
+(
+    nv_alloc_t *at
+)
+{
+    nv_page_pool_t *page_pool = NULL;
+    unsigned int os_pages_in_page = 1 << at->order;
+    unsigned int num_pages = NV_CEIL(at->num_pages, os_pages_in_page);
+    unsigned int i;
+
+    if (at->page_table[0].virt_addr != 0)
+    {
+        // if low on memory, pages could be allocated from different nodes
+        int likely_node_id = page_to_nid(NV_GET_PAGE_STRUCT(at->page_table[0].phys_addr));
+
+        page_pool = nv_mem_pool_get(likely_node_id, at->order);
+    }
+
+    if (at->cache_type != NV_MEMORY_CACHED)
+    {
+        nv_set_memory_type(at, NV_MEMORY_WRITEBACK);
+    }
+
+    for (i = 0; i < num_pages; i++)
+    {
+        nvidia_pte_t *page_ptr = &at->page_table[i * os_pages_in_page];
+
+        if (page_ptr->virt_addr == 0)
+        {
+            // alloc failed
+            break;
+        }
+
+        // For unprotected sysmem in CC, memory is marked as unencrypted during allocation.
+        // NV_FREE_PAGES only deals with protected sysmem. Mark memory as encrypted and protected before free.
+        nv_set_memory_encrypted(at->flags.unencrypted, page_ptr->virt_addr, 1 << at->order);
+    }
+
+    if (!at->flags.pool || page_pool == NULL ||
+        nv_mem_pool_free_pages(page_pool, at) != NV_OK)
+    {
+        // nv_mem_pool_free_pages() fails if !NV_MAY_SLEEP()
+        for (i = 0; i < num_pages; i++)
+        {
+            unsigned int base_os_page = i * os_pages_in_page;
+            nvidia_pte_t *page_ptr = &at->page_table[base_os_page];
+
+            if (page_ptr->virt_addr == 0)
+            {
+                // alloc failed
+                break;
+            }
+
+            NV_FREE_PAGES(page_ptr->virt_addr, at->order);
+        }
+    }
+
+    nv_printf(NV_DBG_MEMINFO,
+            "NVRM: VM: %s: %u/%u order0 pages\n", __FUNCTION__, i * os_pages_in_page, at->num_pages);
+}
+
+NV_STATUS
+nv_alloc_system_pages
+(
+    nv_state_t *nv,
+    nv_alloc_t *at
+)
+{
+    unsigned int gfp_mask = nv_compute_gfp_mask(nv, at);
+    unsigned int i;
+    unsigned int num_pool_allocated_pages = 0;
+    unsigned int os_pages_in_page = 1 << at->order;
+    unsigned int num_pages = NV_CEIL(at->num_pages, os_pages_in_page);
+    // OS allocator tries CPU node first by default, mirror that
+    int preferred_node_id = at->flags.node ? at->node_id : numa_mem_id();
+    nv_page_pool_t *page_pool = nv_mem_pool_get(preferred_node_id, at->order);
+
+    // Remember if pool allocation was attempted and use it on free to avoid hoarding memory
+    // Avoid unwanted scrubbing, especially important for onlined FB
+    // Cross-node cache invalidation at remap can be dramatically slower if memory is not cached locally
+    at->flags.pool = !(gfp_mask & NV_GFP_DMA32) &&
+                     at->flags.zeroed &&
+                     (at->cache_type == NV_MEMORY_CACHED || num_online_nodes() <= 1);
+
+    nv_printf(NV_DBG_MEMINFO,
+            "NVRM: VM: %s: %u order0 pages, %u order\n", __FUNCTION__, at->num_pages, at->order);
+
+    if (page_pool != NULL && at->flags.pool)
+    {
+        num_pool_allocated_pages = nv_mem_pool_alloc_pages(page_pool, at);
+    }
+
+    for (i = num_pool_allocated_pages; i < num_pages; i++)
+    {
+        unsigned long virt_addr = 0;
+
+        if (at->flags.node)
+        {
+            unsigned long ptr = 0ULL;
+            NV_ALLOC_PAGES_NODE(ptr, at->node_id, at->order, gfp_mask);
+            if (ptr != 0)
+            {
+               virt_addr = (unsigned long) page_address((void *)ptr);
+            }
+        }
+        else
+        {
+            NV_GET_FREE_PAGES(virt_addr, at->order, gfp_mask);
+        }
+
+        if (virt_addr == 0)
+        {
+            goto failed;
+        }
+
+        nv_alloc_set_page(at, i, virt_addr);
+    }
+
+    for (i = 0; i < num_pages; i++)
+    {
+        unsigned int base_os_page = i * os_pages_in_page;
+        nvidia_pte_t *page_ptr = &at->page_table[base_os_page];
+        unsigned int num_os_pages = NV_MIN(at->num_pages - base_os_page, os_pages_in_page);
+
+        // In CC, NV_GET_FREE_PAGES only allocates protected sysmem.
+        // To get unprotected sysmem, this memory is marked as unencrypted.
+        nv_set_memory_decrypted_zeroed(at->flags.unencrypted, page_ptr->virt_addr, 1 << at->order,
+                                       num_os_pages * PAGE_SIZE);
+    }
+
+    if (at->cache_type != NV_MEMORY_CACHED)
+    {
+        nv_set_memory_type(at, NV_MEMORY_UNCACHED);
+    }
+
+    return NV_OK;
+
+failed:
+    nv_printf(NV_DBG_MEMINFO,
+        "NVRM: VM: %s: failed to allocate memory\n", __FUNCTION__);
+    nv_free_system_pages(at);
+    return NV_ERR_NO_MEMORY;
 }
 
 NV_STATUS nv_alloc_contig_pages(
@@ -362,32 +578,17 @@ NV_STATUS nv_alloc_contig_pages(
 )
 {
     NV_STATUS status;
-    nvidia_pte_t *page_ptr;
-    NvU32 i, j;
-    unsigned int gfp_mask;
-    unsigned long virt_addr = 0;
-    NvU64 phys_addr;
-    struct device *dev = at->dev;
 
     nv_printf(NV_DBG_MEMINFO,
             "NVRM: VM: %s: %u pages\n", __FUNCTION__, at->num_pages);
 
-    // TODO: This is temporary WAR, we will be removed after fixing bug 200732409.
-    if (os_is_xen_dom0() || os_sev_enabled)
+    if (os_is_xen_dom0())
         return nv_alloc_coherent_pages(nv, at);
 
     at->order = get_order(at->num_pages * PAGE_SIZE);
-    gfp_mask = nv_compute_gfp_mask(nv, at);
 
-    if (at->flags.node0)
-    {
-        NV_ALLOC_PAGES_NODE(virt_addr, 0, at->order, gfp_mask);
-    }
-    else
-    {
-        NV_GET_FREE_PAGES(virt_addr, at->order, gfp_mask);
-    }
-    if (virt_addr == 0)
+    status = nv_alloc_system_pages(nv, at);
+    if (status != NV_OK)
     {
         if (os_is_vgx_hyper())
         {
@@ -402,264 +603,51 @@ NV_STATUS nv_alloc_contig_pages(
             "NVRM: VM: %s: failed to allocate memory\n", __FUNCTION__);
         return NV_ERR_NO_MEMORY;
     }
-#if !defined(__GFP_ZERO)
-    if (at->flags.zeroed)
-        memset((void *)virt_addr, 0, (at->num_pages * PAGE_SIZE));
-#endif
-
-    for (i = 0; i < at->num_pages; i++, virt_addr += PAGE_SIZE)
-    {
-        phys_addr = nv_get_kern_phys_address(virt_addr);
-        if (phys_addr == 0)
-        {
-            nv_printf(NV_DBG_ERRORS,
-                "NVRM: VM: %s: failed to look up physical address\n",
-                __FUNCTION__);
-            status = NV_ERR_OPERATING_SYSTEM;
-            goto failed;
-        }
-
-        page_ptr = at->page_table[i];
-        page_ptr->phys_addr = phys_addr;
-        page_ptr->page_count = NV_GET_PAGE_COUNT(page_ptr);
-        page_ptr->virt_addr = virt_addr;
-        page_ptr->dma_addr = nv_phys_to_dma(dev, page_ptr->phys_addr);
-
-        NV_MAYBE_RESERVE_PAGE(page_ptr);
-    }
-
-    if (at->cache_type != NV_MEMORY_CACHED)
-    {
-        nv_set_contig_memory_type(at->page_table[0],
-                                  at->num_pages,
-                                  NV_MEMORY_UNCACHED);
-    }
-
-    at->flags.coherent = NV_FALSE;
 
     return NV_OK;
-
-failed:
-    if (i > 0)
-    {
-        for (j = 0; j < i; j++)
-            NV_MAYBE_UNRESERVE_PAGE(at->page_table[j]);
-    }
-
-    page_ptr = at->page_table[0];
-    NV_FREE_PAGES(page_ptr->virt_addr, at->order);
-
-    return status;
 }
 
 void nv_free_contig_pages(
     nv_alloc_t *at
 )
 {
-    nvidia_pte_t *page_ptr;
-    unsigned int i;
-
     nv_printf(NV_DBG_MEMINFO,
             "NVRM: VM: %s: %u pages\n", __FUNCTION__, at->num_pages);
 
     if (at->flags.coherent)
         return nv_free_coherent_pages(at);
 
-    if (at->cache_type != NV_MEMORY_CACHED)
-    {
-        nv_set_contig_memory_type(at->page_table[0],
-                                  at->num_pages,
-                                  NV_MEMORY_WRITEBACK);
-    }
-
-    for (i = 0; i < at->num_pages; i++)
-    {
-        page_ptr = at->page_table[i];
-
-        if (NV_GET_PAGE_COUNT(page_ptr) != page_ptr->page_count)
-        {
-            static int count = 0;
-            if (count++ < NV_MAX_RECURRING_WARNING_MESSAGES)
-            {
-                nv_printf(NV_DBG_ERRORS,
-                    "NVRM: VM: %s: page count != initial page count (%u,%u)\n",
-                    __FUNCTION__, NV_GET_PAGE_COUNT(page_ptr),
-                    page_ptr->page_count);
-            }
-        }
-        NV_MAYBE_UNRESERVE_PAGE(page_ptr);
-    }
-
-    page_ptr = at->page_table[0];
-
-    NV_FREE_PAGES(page_ptr->virt_addr, at->order);
+    nv_free_system_pages(at);
 }
 
-NV_STATUS nv_alloc_system_pages(
-    nv_state_t *nv,
-    nv_alloc_t *at
-)
+static NvUPtr nv_vmap(struct page **pages, NvU32 page_count,
+                      NvBool cached, NvBool unencrypted)
 {
-    NV_STATUS status;
-    nvidia_pte_t *page_ptr;
-    NvU32 i, j;
-    unsigned int gfp_mask;
-    unsigned long virt_addr = 0;
-    NvU64 phys_addr;
-    struct device *dev = at->dev;
-    dma_addr_t bus_addr;
-
-    nv_printf(NV_DBG_MEMINFO,
-            "NVRM: VM: %u: %u pages\n", __FUNCTION__, at->num_pages);
-
-    gfp_mask = nv_compute_gfp_mask(nv, at);
-
-    for (i = 0; i < at->num_pages; i++)
+    void *ptr;
+    pgprot_t prot = PAGE_KERNEL;
+#if defined(NVCPU_X86_64)
+    if (unencrypted)
     {
-        if (os_sev_enabled && (dev != NULL))
-        {
-            virt_addr = (unsigned long)dma_alloc_coherent(dev,
-                                                          PAGE_SIZE,
-                                                          &bus_addr,
-                                                          gfp_mask);
-            at->flags.coherent = NV_TRUE;
-            //
-            // dma_alloc_coherent() allocates unencrypted memory when AMD SEV
-            // is enabled.
-            //
-            at->flags.unencrypted = NV_TRUE;
-        }
-        else if (at->flags.node0)
-        {
-            NV_ALLOC_PAGES_NODE(virt_addr, 0, 0, gfp_mask);
-        }
-        else
-        {
-            NV_GET_FREE_PAGES(virt_addr, 0, gfp_mask);
-        }
-
-        if (virt_addr == 0)
-        {
-            nv_printf(NV_DBG_MEMINFO,
-                "NVRM: VM: %s: failed to allocate memory\n", __FUNCTION__);
-            status = NV_ERR_NO_MEMORY;
-            goto failed;
-        }
-#if !defined(__GFP_ZERO)
-        if (at->flags.zeroed)
-            memset((void *)virt_addr, 0, PAGE_SIZE);
-#endif
-
-        phys_addr = nv_get_kern_phys_address(virt_addr);
-        if (phys_addr == 0)
-        {
-            nv_printf(NV_DBG_ERRORS,
-                "NVRM: VM: %s: failed to look up physical address\n",
-                __FUNCTION__);
-            NV_FREE_PAGES(virt_addr, 0);
-            status = NV_ERR_OPERATING_SYSTEM;
-            goto failed;
-        }
-
-#if defined(_PAGE_NX)
-        if (((_PAGE_NX & pgprot_val(PAGE_KERNEL)) != 0) &&
-                (phys_addr < 0x400000))
-        {
-            nv_printf(NV_DBG_SETUP,
-                "NVRM: VM: %s: discarding page @ 0x%llx\n",
-                __FUNCTION__, phys_addr);
-            --i;
-            continue;
-        }
-#endif
-
-        page_ptr = at->page_table[i];
-        page_ptr->phys_addr = phys_addr;
-        page_ptr->page_count = NV_GET_PAGE_COUNT(page_ptr);
-        page_ptr->virt_addr = virt_addr;
-
-        //
-        // Use unencrypted dma_addr returned by dma_alloc_coherent() as
-        // nv_phys_to_dma() returns encrypted dma_addr when AMD SEV is enabled.
-        //
-        if (at->flags.coherent)
-            page_ptr->dma_addr = bus_addr;
-        else if (dev)
-            page_ptr->dma_addr = nv_phys_to_dma(dev, page_ptr->phys_addr);
-        else
-            page_ptr->dma_addr = page_ptr->phys_addr;
-
-        NV_MAYBE_RESERVE_PAGE(page_ptr);
+        prot = cached ? nv_adjust_pgprot(PAGE_KERNEL_NOENC) :
+                        nv_adjust_pgprot(NV_PAGE_KERNEL_NOCACHE_NOENC);
     }
-
-    if (at->cache_type != NV_MEMORY_CACHED)
-        nv_set_memory_type(at, NV_MEMORY_UNCACHED);
-
-    return NV_OK;
-
-failed:
-    if (i > 0)
+    else
     {
-        for (j = 0; j < i; j++)
-        {
-            page_ptr = at->page_table[j];
-            NV_MAYBE_UNRESERVE_PAGE(page_ptr);
-            if (at->flags.coherent)
-            {
-                dma_free_coherent(dev, PAGE_SIZE, (void *)page_ptr->virt_addr,
-                                  page_ptr->dma_addr);
-            }
-            else
-            {
-                NV_FREE_PAGES(page_ptr->virt_addr, 0);
-            }
-        }
+        prot = cached ? PAGE_KERNEL : PAGE_KERNEL_NOCACHE;
     }
+#elif defined(NVCPU_AARCH64)
+    prot = cached ? PAGE_KERNEL : NV_PGPROT_UNCACHED(PAGE_KERNEL);
+#endif
+    ptr = vmap(pages, page_count, VM_MAP, prot);
+    NV_MEMDBG_ADD(ptr, page_count * PAGE_SIZE);
 
-    return status;
+    return (NvUPtr)ptr;
 }
 
-void nv_free_system_pages(
-    nv_alloc_t *at
-)
+static void nv_vunmap(NvUPtr vaddr, NvU32 page_count)
 {
-    nvidia_pte_t *page_ptr;
-    unsigned int i;
-    struct device *dev = at->dev;
-
-    nv_printf(NV_DBG_MEMINFO,
-            "NVRM: VM: %s: %u pages\n", __FUNCTION__, at->num_pages);
-
-    if (at->cache_type != NV_MEMORY_CACHED)
-        nv_set_memory_type(at, NV_MEMORY_WRITEBACK);
-
-    for (i = 0; i < at->num_pages; i++)
-    {
-        page_ptr = at->page_table[i];
-
-        if (NV_GET_PAGE_COUNT(page_ptr) != page_ptr->page_count)
-        {
-            static int count = 0;
-            if (count++ < NV_MAX_RECURRING_WARNING_MESSAGES)
-            {
-                nv_printf(NV_DBG_ERRORS,
-                    "NVRM: VM: %s: page count != initial page count (%u,%u)\n",
-                    __FUNCTION__, NV_GET_PAGE_COUNT(page_ptr),
-                    page_ptr->page_count);
-            }
-        }
-
-        NV_MAYBE_UNRESERVE_PAGE(page_ptr);
-        if (at->flags.coherent)
-        {
-            dma_free_coherent(dev, PAGE_SIZE, (void *)page_ptr->virt_addr,
-                              page_ptr->dma_addr);
-        }
-        else
-        {
-            NV_FREE_PAGES(page_ptr->virt_addr, 0);
-        }
-    }
+    vunmap((void *)vaddr);
+    NV_MEMDBG_REMOVE((void *)vaddr, page_count * PAGE_SIZE);
 }
 
 NvUPtr nv_vm_map_pages(
@@ -700,30 +688,3 @@ void nv_vm_unmap_pages(
 
     nv_vunmap(virt_addr, count);
 }
-
-void nv_address_space_init_once(struct address_space *mapping)
-{
-#if defined(NV_ADDRESS_SPACE_INIT_ONCE_PRESENT)
-    address_space_init_once(mapping);
-#else
-    memset(mapping, 0, sizeof(*mapping));
-    INIT_RADIX_TREE(&mapping->page_tree, GFP_ATOMIC);
-
-#if defined(NV_ADDRESS_SPACE_HAS_RWLOCK_TREE_LOCK)
-    //
-    // The .tree_lock member variable was changed from type rwlock_t, to
-    // spinlock_t, on 25 July 2008, by mainline commit
-    // 19fd6231279be3c3bdd02ed99f9b0eb195978064.
-    //
-    rwlock_init(&mapping->tree_lock);
-#else
-    spin_lock_init(&mapping->tree_lock);
-#endif
-
-    spin_lock_init(&mapping->i_mmap_lock);
-    INIT_LIST_HEAD(&mapping->private_list);
-    spin_lock_init(&mapping->private_lock);
-    INIT_RAW_PRIO_TREE_ROOT(&mapping->i_mmap);
-    INIT_LIST_HEAD(&mapping->i_mmap_nonlinear);
-#endif /* !NV_ADDRESS_SPACE_INIT_ONCE_PRESENT */
-}
