@@ -341,7 +341,7 @@ NvBool NV_API_CALL os_semaphore_may_sleep(void)
 
 NvBool NV_API_CALL os_is_isr(void)
 {
-    return (in_irq());
+    return (nv_in_hardirq());
 }
 
 // return TRUE if the caller is the super-user
@@ -493,7 +493,7 @@ NV_STATUS NV_API_CALL os_memcpy_from_user(
     NvU32       n
 )
 {
-    return (NV_COPY_FROM_USER(to, from, n) ? NV_ERR_INVALID_ADDRESS : NV_OK);
+    return (copy_from_user(to, from, n) ? NV_ERR_INVALID_ADDRESS : NV_OK);
 }
 
 NV_STATUS NV_API_CALL os_memcpy_to_user(
@@ -502,7 +502,7 @@ NV_STATUS NV_API_CALL os_memcpy_to_user(
     NvU32       n
 )
 {
-    return (NV_COPY_TO_USER(to, from, n) ? NV_ERR_INVALID_ADDRESS : NV_OK);
+    return (copy_to_user(to, from, n) ? NV_ERR_INVALID_ADDRESS : NV_OK);
 }
 
 void* NV_API_CALL os_mem_set(
@@ -723,7 +723,7 @@ NvU64 NV_API_CALL os_get_cpu_frequency(void)
 
 NvU32 NV_API_CALL os_get_current_process(void)
 {
-    return NV_GET_CURRENT_PROCESS();
+    return current->tgid;
 }
 
 void NV_API_CALL os_get_current_process_name(char *buf, NvU32 len)
@@ -1200,7 +1200,7 @@ NvU32 NV_API_CALL os_get_cpu_number(void)
 
 NvU32 NV_API_CALL os_get_cpu_count(void)
 {
-    return NV_NUM_CPUS();
+    return num_possible_cpus();
 }
 
 NvBool NV_API_CALL os_pat_supported(void)
@@ -1629,7 +1629,15 @@ NV_STATUS NV_API_CALL os_alloc_pages_node
      *                              the requested order is too large (just fail
      *                              instead).
      *
-     * 5. (Optional) __GFP_RECLAIM: Used to allow/forbid reclaim.
+     * 5. __GFP_RETRY_MAYFAIL:      Used to avoid the Linux kernel OOM killer.
+     *                              To help PMA on paths where UVM might be in
+     *                              memory over subscription. This gives UVM a
+     *                              chance to free memory before invoking any
+     *                              action from the OOM killer.  Freeing
+     *                              non-essential memory will also benefit the
+     *                              system as a whole.
+     *
+     * 6. (Optional) __GFP_RECLAIM: Used to allow/forbid reclaim.
      *                              This is part of GFP_USER and consequently
      *                              GFP_HIGHUSER_MOVABLE.
      *
@@ -1643,37 +1651,12 @@ NV_STATUS NV_API_CALL os_alloc_pages_node
      */
 
     gfp_mask = __GFP_THISNODE | GFP_HIGHUSER_MOVABLE | __GFP_COMP |
-               __GFP_NOWARN;
-    
-#if defined(__GFP_RETRY_MAYFAIL)
+               __GFP_NOWARN | __GFP_RETRY_MAYFAIL;
 
-    /*
-     * __GFP_RETRY_MAYFAIL :  Used to avoid the Linux kernel OOM killer.
-     *                        To help PMA on paths where UVM might be
-     *                        in memory over subscription. This gives UVM 
-     *                        a chance to free memory before invoking any 
-     *                        action from the OOM killer.
-     *                        Freeing non-essential memory will also benefit 
-     *                        the system as a whole.
-     */
-
-    gfp_mask |= __GFP_RETRY_MAYFAIL;
-#elif defined(__GFP_NORETRY)
-
-    /*
-     *  __GFP_NORETRY :       Use __GFP_NORETRY on older kernels where
-     *                        __GFP_RETRY_MAYFAIL is not present.
-     */
-
-    gfp_mask |= __GFP_NORETRY;
-#endif
-
-#if defined(__GFP_RECLAIM)
     if (flag & NV_ALLOC_PAGES_NODE_SKIP_RECLAIM)
     {
         gfp_mask &= ~(__GFP_RECLAIM);
     }
-#endif // defined(__GFP_RECLAIM)
 
     alloc_addr = alloc_pages_node(nid, gfp_mask, order);
     if (alloc_addr == NULL)
@@ -2179,6 +2162,16 @@ NvS32 NV_API_CALL os_imex_channel_get
     return nv_caps_imex_channel_get((int)descriptor);
 }
 
+NV_STATUS NV_API_CALL os_tegra_igpu_perf_boost
+(
+    void *handle,
+    NvBool enable,
+    NvU32 duration
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
 /*
  * Reads the total memory and free memory of a NUMA node from the kernel.
  */
