@@ -1,15 +1,27 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 1999-2020 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #define  __NO_VERSION__
-#include "nv-misc.h"
 
 #include "os-interface.h"
 #include "nv-linux.h"
@@ -34,7 +46,7 @@ void* NV_API_CALL os_pci_init_handle(
     {
         if (vendor) *vendor = dev->vendor;
         if (device) *device = dev->device;
-        pci_dev_put(dev); /* XXX Fix me! (hotplug) */
+        pci_dev_put(dev); /* TODO: Fix me! (hotplug) */
     }
     return (void *) dev;
 }
@@ -125,70 +137,64 @@ NV_STATUS NV_API_CALL os_pci_write_dword(
 
 NvBool NV_API_CALL os_pci_remove_supported(void)
 {
-#if defined NV_PCI_STOP_AND_REMOVE_BUS_DEVICE
     return NV_TRUE;
-#else
-    return NV_FALSE;
-#endif
 }
 
 void NV_API_CALL os_pci_remove(
     void *handle
 )
 {
-#if defined(NV_PCI_STOP_AND_REMOVE_BUS_DEVICE)
-    NV_PCI_STOP_AND_REMOVE_BUS_DEVICE(handle);
-#elif defined(DEBUG)
-    nv_printf(NV_DBG_ERRORS,
-            "NVRM: %s() is called even though NV_PCI_STOP_AND_REMOVE_BUS_DEVICE is not defined\n",
-            __FUNCTION__);
-    os_dbg_breakpoint();
-#endif
+    pci_stop_and_remove_bus_device(handle);
 }
 
+NV_STATUS NV_API_CALL
+os_enable_pci_req_atomics(
+    void *handle,
+    enum os_pci_req_atomics_type type
+)
+{
+#ifdef NV_PCI_ENABLE_ATOMIC_OPS_TO_ROOT_PRESENT
+    int ret;
+    u16 val;
 
+    switch (type)
+    {
+        case OS_INTF_PCIE_REQ_ATOMICS_32BIT:
+            ret = pci_enable_atomic_ops_to_root(handle,
+                                                PCI_EXP_DEVCAP2_ATOMIC_COMP32);
+            break;
+        case OS_INTF_PCIE_REQ_ATOMICS_64BIT:
+            ret = pci_enable_atomic_ops_to_root(handle,
+                                                PCI_EXP_DEVCAP2_ATOMIC_COMP64);
+            break;
+        case OS_INTF_PCIE_REQ_ATOMICS_128BIT:
+            ret = pci_enable_atomic_ops_to_root(handle,
+                                                PCI_EXP_DEVCAP2_ATOMIC_COMP128);
+            break;
+        default:
+            ret = -1;
+            break;
+    }
 
+    if (ret == 0)
+    {
+        /*
+         * GPUs that don't support Requester Atomics have its
+         * PCI_EXP_DEVCTL2_ATOMIC_REQ always set to 0 even after SW enables it.
+         */
+        if ((pcie_capability_read_word(handle, PCI_EXP_DEVCTL2, &val) == 0) &&
+            (val & PCI_EXP_DEVCTL2_ATOMIC_REQ))
+        {
+            return NV_OK;
+        }
+    }
+#endif
+    return NV_ERR_NOT_SUPPORTED;
+}
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+void NV_API_CALL os_pci_trigger_flr(void *handle)
+{
+    nv_printf(NV_DBG_ERRORS,
+        "NVRM: %s() PCI FLR skipped\n", __FUNCTION__);
+    return;
+}
